"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const FuzzySet = require("fuzzyset.js");
class Analyzer {
    constructor() {
        this.featureRegExp = new RegExp('^(feature)+[:]?', 'i');
        this.scenarioOutlineRegExp = new RegExp('^(scenario+[ ]+outline)+[:]?', 'i');
        this.scenarioRegExp = new RegExp('^(scenario)+[:]?', 'i');
        this.stateRegExp = new RegExp('^(given)+[:]?', 'i');
        this.actionRegExp = new RegExp('^(when)+[:]?', 'i');
        this.outcomeRegExp = new RegExp('^(then)+[:]?', 'i');
        this.glueRegExp = new RegExp('^(and|but)+[:]?', 'i');
        this.examplesTitleRegExp = new RegExp('^(examples|example)', 'i');
        this.examplesDataRegExp = new RegExp('^[|]', 'i');
    }
    getGherkins(listRows, index) {
        const features = [];
        const scenarios = [];
        const states = [];
        const actions = [];
        const outcomes = [];
        const files = [];
        let rowAnalyzed;
        let currentId = '';
        let currentReadRow = '';
        for (const listRow of listRows) {
            if (rowAnalyzed = this.getValidFeature(listRow)) {
                currentReadRow = 'feature';
                currentId = `fe-${index}-${features.length}`;
                features.push({ id: currentId, text: rowAnalyzed });
            }
            else if (rowAnalyzed = this.getValidScenarioOutline(listRow)) {
                currentReadRow = 'scenario';
                currentId = `sc-${index}-${scenarios.length}`;
                scenarios.push({ id: currentId, text: rowAnalyzed });
            }
            else if (rowAnalyzed = this.getValidScenario(listRow)) {
                currentReadRow = 'scenario';
                currentId = `sc-${index}-${scenarios.length}`;
                scenarios.push({ id: currentId, text: rowAnalyzed });
            }
            else if (rowAnalyzed = this.getValidState(listRow, currentReadRow)) {
                currentReadRow = 'state';
                currentId = `st-${index}-${states.length}`;
                states.push({ id: currentId, text: rowAnalyzed });
            }
            else if (rowAnalyzed = this.getValidAction(listRow, currentReadRow)) {
                currentReadRow = 'action';
                currentId = `ac-${index}-${actions.length}`;
                actions.push({ id: currentId, text: rowAnalyzed });
            }
            else if (rowAnalyzed = this.getValidOutcome(listRow, currentReadRow)) {
                currentReadRow = 'outcome';
                currentId = `ou-${index}-${outcomes.length}`;
                outcomes.push({ id: currentId, text: rowAnalyzed });
            }
            else if (rowAnalyzed = this.getValidExampleTitle(listRow)) {
                currentReadRow = 'example';
                currentId = '';
            }
            else if (rowAnalyzed = this.getValidExampleData(listRow, currentReadRow)) {
                currentReadRow = 'example';
                currentId = '';
            }
            else {
                currentId = '';
                continue;
            }
            files.push({ id: currentId, text: listRow });
        }
        return { files, features, scenarios, states, actions, outcomes };
    }
    getSimilarities(gherkinsTexts, gherkinsObjects) {
        const similarities = {};
        for (let index = 0; index < gherkinsObjects.length; index++) {
            const totalGherkins = [...gherkinsTexts];
            totalGherkins.splice(index, 1);
            const fuzzySetHandler = FuzzySet(totalGherkins, false);
            const fuzzySetReport = fuzzySetHandler.get(gherkinsObjects[index].text, [], 0.75);
            if (fuzzySetReport && fuzzySetReport.length) {
                const gherkinId = gherkinsObjects[index].id;
                similarities[gherkinId] = {};
                similarities[gherkinId].class = (fuzzySetReport[0][0] === 1) ? 'equal' : 'similar';
                similarities[gherkinId].similarities = this.getListSimilarities(fuzzySetReport, gherkinsObjects, gherkinId);
            }
        }
        return similarities;
    }
    getListSimilarities(report, gherkins, gherkinId) {
        const similarGherkins = report.map((elem) => elem[1]);
        return gherkins
            .filter((elem) => elem.id !== gherkinId)
            .filter((elem) => similarGherkins.includes(elem.text))
            .map((elem) => ({ id: elem.id, type: this.getSimilarityLinkType(elem.id), text: elem.text }));
    }
    getSimilarityLinkType(similarityId) {
        const similarityPrefix = similarityId.substring(0, 2);
        let similarityType;
        switch (similarityPrefix) {
            case 'fe':
                similarityType = 'features';
                break;
            case 'sc':
                similarityType = 'scenarios';
                break;
            case 'st':
                similarityType = 'states';
                break;
            case 'ac':
                similarityType = 'actions';
                break;
            case 'ou':
                similarityType = 'outcomes';
                break;
            default:
                similarityType = '';
                break;
        }
        return similarityType;
    }
    getValidFeature(listRow) {
        const listRowToAnalyze = this.featureRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidScenarioOutline(listRow) {
        const listRowToAnalyze = this.scenarioOutlineRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidScenario(listRow) {
        const listRowToAnalyze = this.scenarioRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidState(listRow, previousRow) {
        const listRowToAnalyze = previousRow === 'state' ? this.glueRegExp.exec(listRow) : this.stateRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidAction(listRow, previousRow) {
        const listRowToAnalyze = previousRow === 'action' ? this.glueRegExp.exec(listRow) : this.actionRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidOutcome(listRow, previousRow) {
        const listRowToAnalyze = previousRow === 'outcome' ? this.glueRegExp.exec(listRow) : this.outcomeRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidExampleTitle(listRow) {
        const listRowToAnalyze = this.examplesTitleRegExp.exec(listRow);
        return listRowToAnalyze ? listRowToAnalyze[0].trim() : null;
    }
    getValidExampleData(listRow, previousRow) {
        if (previousRow === 'example') {
            const listRowToAnalyze = this.examplesDataRegExp.exec(listRow);
            return listRowToAnalyze ? listRow.trim() : null;
        }
        return null;
    }
}
exports.Analyzer = Analyzer;
